
--[[

name:   Antix Animator
vers:   1.0.1
desc:   A simple frame based animation system
auth:   by Cliff Earl
date:   March 2016
legal:  (c) 2016 Antix Software

hist:   v1.0.1 - Added frameskipping to avoid wasteful calls to setRegion(). Thanks to keszegh

--]]

Animator = Core.class()

-- NOTHING MUCH HAPPENS ON STARTUP
function Animator:init()
end

-- ASSIGN ANIMATIONS TO ACTOR
function Animator:attach(actor, a)
  actor.anim_regions    = a.regions
  actor.animations      = a.animations
  actor.anim_paused     = true
  actor.anim_complete   = false
  actor.anim_speed      = 0
  actor.anim_loops      = false
  actor.anim_timer      = 0
  actor.anim_frames     = nil
  actor.anim_length     = 0
  actor.anim_prevFrame  = 0
end

-- DETACH ANIMATIONS FROM ACTOR
function Animator:detach(actor)
  actor.anim_regions    = nil
  actor.animations      = nil
  actor.anim_paused     = nil
  actor.anim_complete   = nil
  actor.anim_speed      = nil
  actor.anim_loops      = nil
  actor.anim_timer      = nil
  actor.anim_frames     = nil
  actor.anim_length     = nil
  actor.anim_prevFrame  = nil
end

-- UTILITY FUNCTION - DESTROY ANIMATION
function Animator:dispose(table)
  for k in next,table do
    local t = type(k)
    if k == "table" then -- IF NEXT IS A TABLE THEN EMPTY THAT FIRST
      self:dispose(k)
    end
    table[k] = nil
  end
end

-- SET WHICH ANIMATION TO PLAY
function Animator:set(actor, animation)
  local a = actor.animations[animation]
  actor.anim_frames     = a.frames
  actor.anim_speed      = a.speed
  actor.anim_length     = a.length
  actor.anim_complete   = false
  actor.anim_paused     = false
  actor.anim_loops      = a.loops
  actor.anim_timer      = 0
  actor.anim_prevFrame  = -1
end

-- SET ACTORS ANIMATION PLAYING OR PAUSED
function Animator:pause(actor, state)
  actor.anim_paused = state
end

-- ANIMATE ACTOR
function Animator:update(actor, dt)
  if actor.anim_paused then return end -- DON'T BOTHER IF IT'S PAUSED
  
  local floor = math.floor
  local function clamp(n, min, max) return n < min and min or (n > max and max or n) end
  
  if actor.anim_loops then -- PLAY LOOPING ANIMATION
    local timer = actor.anim_timer + (dt * actor.anim_speed)
    local nextFrame = clamp(floor(actor.anim_length * timer + 1), 1, actor.anim_length)
      if nextFrame ~= actor.anim_prevFrame then
        local region = actor.region
        local r = actor.anim_regions[actor.anim_frames[nextFrame]]
        region:setRegion(r[1], r[2], r[3], r[4]) -- SET NEW TEXTURE REGION
        actor:setTextureRegion(region)
        actor.anim_prevFrame = nextFrame
      end
    if timer >= 1 then timer = timer - 1 end
    actor.anim_timer = timer
  else
    if not actor.anim_complete then -- PLAY NON LOOPING ANIMATION
      local timer = actor.anim_timer + (dt * actor.anim_speed)
      local nextFrame = clamp(floor(actor.anim_length * timer + 1), 1, actor.anim_length)
      if nextFrame ~= actor.anim_prevFrame then
        local region = actor.region
        local r = actor.anim_regions[actor.anim_frames[nextFrame]]
        region:setRegion(r[1], r[2], r[3], r[4])
        actor:setTextureRegion(region) -- SET NEW TEXTURE REGION
        actor.anim_prevFrame = nextFrame
      end
      if timer >= 1 then actor.anim_complete = true end
      actor.anim_timer = timer
    end
  end
end
