
--[[

name:   Antix Rect Button
vers:   1.0
desc:   A simple rectangular button that performs an action on release
auth:   by Cliff Earl
date:   March 2016
legal:  (c) 2016 Antix Software

--]]

rectButton = Core.class(Shape)

function rectButton:init(x, y, w, h, color, caption, textColor, action)
  self.enabled = true
  self.focus = false
  self.caption = caption or ""
  
  self:setFillStyle(Shape.SOLID, color) -- DRAW THE SHAPE
  self:beginPath()
  self:moveTo(0, 0)
  self:lineTo(w, 0)
  self:lineTo(w, h)
  self:lineTo(0, h)
  self:closePath()
  self:endPath()
  self:setPosition(x, y)
  
  local text = TextField.new(nil, caption) -- CREATE CAPTION
  text:setScale(2)
  text:setTextColor(textColor)
  text:setPosition((w - text:getWidth()) * 0.5, h - text:getHeight() * 0.5 - 4)
  self:addChild(text)
  
  self.action = action or nil -- FUNCTION TO CALL WHEN BUTTON IS RELEASED
  
  self:addEventListener(Event.TOUCHES_BEGIN, self.pressed, self) -- ADD EVENT HANDLERS
  self:addEventListener(Event.TOUCHES_END, self.released, self)
end

function rectButton:destroy()
  self.enabled = false
  
  self:removeEventListener(Event.TOUCHES_BEGIN, self.pressed, self)
  self:removeEventListener(Event.TOUCHES_END, self.released, self)
  
  self:removeChildAt(1) -- GET RID OF CAPTION
  
  self:clear() -- CLEAR SHAPE
  
  self.action = nil
  
  self.caption = nil
  self.enabled = nil
  self.focus = nil
end

function rectButton:enable(state)
  self.enabled = state
end

function rectButton:pressed(e)
  if not self:isVisible() or not self.enabled then return end
	if self:hitTestPoint(e.touch.x, e.touch.y) then
    self.focus = true
    e:stopPropagation()
  end
end

function rectButton:released(e)
  if not self:isVisible() or not self.enabled then return end
	if self.focus and self:hitTestPoint(e.touch.x, e.touch.y) then
    if self.focus then
      self.focus = false
      e:stopPropagation()
      if self.action then
        
        self.action(self) -- PERFORM ACTION
        
      end
    end
  end
end
