
--[[

name:   Antix Animation system
vers:   1.0
desc:   Example of my simple animation system
auth:   by Cliff Earl
date:   March 2016
legal:  (c) 2016 Antix Software

--]]

--require("mobdebug").start() -- ONLY OF INTEREST TO ZEROBRANE USERS

require("json")

-- SET APPLICATION STUFF
local currentTimer = 0
local I_WIDTH = application:getContentWidth ()
local I_HEIGHT = application:getContentHeight()
application:setFps(30)
application:setBackgroundColor(0x000000)
stage:setOrientation(Stage.PORTRAIT)
stage:setClip(0, 0, I_WIDTH, I_HEIGHT)

local File = File.new() -- FOR FILE OPERATIONS

local Animator = Animator.new() -- THE ANIMATOR CLASS!!

-- LOAD PLAYER ATLAS AND CREATE PLAYER ACTOR
local playerAtlas = Texture.new("images/player.png")
local player = Actor.new(TextureRegion.new(playerAtlas, 0, 0, 0, 0))
player:setPosition((I_WIDTH - 64) * 0.5, 32)
stage:addChild(player)

-- LOAD ANIMATIONS, ATTACH THEM TO PLAYER, AND PLAY IDLE ANIMATION
local playerAnimations = File:loadJSON("images/player.json")
Animator:attach(player, playerAnimations)
Animator:set(player, "idle")

-- PLAY SELECTED ANIMATION
local function playAnimation(button)
  Animator:set(player, button.caption)
end

-- CREATE BUTTONS THAT WILL PLAY DIFFERENT ANIMATIONS WHEN PRESSED
local buttons = Sprite.new()
buttons:addChild(rectButton.new( 10, 130, 145, 40, 0x446688, "idle", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 130, 145, 40, 0x446688, "balance", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new( 10, 180, 145, 40, 0x446688, "hurt", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 180, 145, 40, 0x446688, "die", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new( 10, 230, 145, 40, 0x446688, "walk", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 230, 145, 40, 0x446688, "run", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new( 10, 280, 145, 40, 0x446688, "wallslide", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 280, 145, 40, 0x446688, "push", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new( 10, 330, 145, 40, 0x446688, "kick", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 330, 145, 40, 0x446688, "float", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new( 10, 380, 145, 40, 0x446688, "swim1", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 380, 145, 40, 0x446688, "swim2", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new( 10, 430, 145, 40, 0x446688, "throw1", 0xddeeff, playAnimation))
buttons:addChild(rectButton.new(165, 430, 145, 40, 0x446688, "throw2", 0xddeeff, playAnimation))
stage:addChild(buttons)

-- MAIN PROGRAM LOOP
local function onEnterFrame(event)
	local timer = os.timer()
	local dt = timer - currentTimer
	currentTimer = timer
  
  Animator:update(player, dt) -- UPDATE PLAYER ANIMATION
  
end

stage:addEventListener(Event.ENTER_FRAME, onEnterFrame)
